function net = MLP_digits_demo(varargin)
%% A Multi-Layer-Perceptron demo with handwritten digit recognition.
% usage:  net = MLP_digits_demo(); to create a new network
%         net = MLP_digits_demo('network_filename'); to use an existing network
% Author: Nico Schmidt, Mar 2011
%
% Training data taken from here:
% http://archive.ics.uci.edu/ml/datasets/Optical+Recognition+of+Handwritten+Digits
% Direct link to optdigits.tra:
% http://archive.ics.uci.edu/ml/machine-learning-databases/optdigits/optdigits.tra
% Save optdigits.tra in same folder as this script.
%
%% define parameters:

Ndata = 3823;             % size of the training set (3823 contained in optdigits.tra)
Nneurons = [30];          % number of neurons in hidden layer(s) [nLayer1 nLayer2 ...]
activation_fnc = {'tansig', 'tansig'};  % activation function of layers
goal_error = 0.001;       % stop criterion (error rate which we want to reach)
goal_gradient = 1e-30;    % stop criterion (gradient at which we stop)
max_failures = 30;        % stop criterion (Maximum number of validation failures)
Nepochs = 1000;           % maximum number of training epochs
learningrate = 0.1;       % initial learning rate
learningrate_inc = 1.05;  % Ratio to increase learning rate
learningrate_dec = 0.7;   % Ratio to decrease learning rate


%% load data:
optdigits = load('optdigits.tra');

Ndata = min(Ndata,size(optdigits,1));
targets = -ones(10,Ndata);
targets((0:Ndata-1)' .*10  + 1 + optdigits(1:Ndata,end)) = 1;
optdigits = optdigits(1:Ndata,1:end-1)';

%% plot a few exaples:
figure(1), clf
set(1, 'Name','Training Data', ...
       'Menubar', 'none', ...
       'Toolbar', 'none')
colormap gray
for ii=1:100
  subplot(10,10,ii)
  imagesc(reshape(optdigits(:,ii),[8 8])')
  set(gca, 'XTick', [], 'YTick', [])
end


%% train the network:
if nargin==0
  net = newff(optdigits,targets,Nneurons,activation_fnc,'traingda');
  net.trainParam.goal = goal_error;
  net.trainParam.min_grad = goal_gradient;
  net.trainParam.max_fail = max_failures;
  net.trainParam.epochs = Nepochs;
  net.trainParam.lr = learningrate;
  net.trainParam.lr_inc = learningrate_inc;
  net.trainParam.lr_dec = learningrate_dec;
  net = train(net,optdigits,targets);
  save('net','net')
else
  load(varargin{1});
  net = train(net,optdigits,targets);
end

%% test your network:
figure(2), clf
set(2, 'Name', 'Test New Samples', ...
       'Units', 'normalized', ...
       'Menubar', 'none', ...
       'Toolbar', 'none')
ax2 = axes('Position', [0.9 0 0.1 1]);
set(ax2, 'Visible', 'off')
ax1 = axes('Position', [0 0 0.8 1]);
colormap gray
hImage = imagesc(zeros(8,8), [0 16]);
for ii=0:9
  text('Units', 'normalized', 'String', sprintf('%d', ii), ...
       'Position', [1.075, 0.95-(ii/10)], ...
       'Fontweight', 'bold', 'FontSize', 12);
end
set(hImage,'HitTest','off')
set(ax1, 'XTick', [], 'YTick', [])
clear persistent
set(gcf,'WindowButtonDownFcn', {@move, ax1, ax2, hImage, 1, net}, ...
        'WindowButtonMotionFcn', {@move, ax1, ax2, hImage, 2, net}, ...
        'WindowButtonUpFcn', {@move, ax1, ax2, hImage, 3, net})

end
      
      
      
function move(hFigure, ~, ax1, ax2, hImage, btn, net)

persistent moving dat hScore

img_size = 8;
step_size = 1;
nFields = img_size * step_size;

pos_ax = get(ax1, 'Position');
pos_mouse = (get(hFigure, 'CurrentPoint') - pos_ax(1:2));
if (pos_mouse(1) > 0 && pos_mouse(1) < pos_ax(3) && ...
    pos_mouse(2) > 0 && pos_mouse(2) < pos_ax(4))
  if btn==1 && (isempty(moving) || ~moving)
    moving = 1;
    dat = zeros(nFields, nFields);
    if strcmp(get(ax2, 'Visible'), 'on'), set(ax2, 'Visible', 'off'), end
    try delete(hScore), end
  end
  
  if ismember(btn, [1 2])
    if moving
      %% draw with mouse:
      pos_data = ceil(pos_mouse./pos_ax(3:4) * nFields);
      dat(pos_data(2),pos_data(1)) = 16 / (step_size*step_size);
      img_data = zeros(img_size,img_size);
      for xx=1:size(img_data,1)
        for yy=1:size(img_data,2)
          img_data(xx,yy) = sum(sum(dat(((xx-1)*step_size+1):(xx*step_size), ((yy-1)*step_size+1):(yy*step_size))));
        end
      end      
      set(hImage, 'CData', flipud(img_data))
    end
  end
  
  if btn==3
    %% feed image into network:
    moving = 0;
    dat = zeros(nFields, nFields);
    img_data = reshape(get(hImage, 'CData')', [img_size*img_size, 1]);
    out = sim(net,img_data); % here the network evaluates the input

    title(mat2str(out, 2))
    axes(ax2)
    set(ax2, 'Visible', 'on')
    hScore = imagesc(out, [-1 1]);
    set(ax2, 'XTick', [], 'YTick', [])
  end    
end

end
